// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = wrap;
    AddressV = wrap;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

float xStart;
int MaxIter = 32;
int WarmUp = 16;
float4x4 sequence;

// Compute single Lyapunov lambda value
float lyapunovExponent (const float2 ab)
  {

    double r;
    double sum = 0;
    double x = xStart;
    int k;
    
    for(k = 0; k < MaxIter+WarmUp; k++)
    {
      r = lerp(ab.x, ab.y, sequence[fmod(k*0.25, 4)][fmod(k, 4)]);

      x = r * x * ( 1-x );

      if (k >= WarmUp)
         sum += log( r - 2*r*x );

    }

    return clamp(sum/(k-WarmUp), -0.99, 0);

  }//LyapunovExponent
  
  //PS 2.b version
  float lyapunovExponentPS2 (const float2 ab)
  {

    double r;
    double sum = 0;
    double x = xStart;
    int k;
    WarmUp = 32;
    MaxIter = 64;

    for(k = 0; k < MaxIter+WarmUp; k++)
    {
      r = lerp(ab.x, ab.y, sequence[fmod(k*0.25, 4)][fmod(k, 4)]);

      x = r * x * ( 1-x );

      sum += k >= WarmUp ? 0 : log( r - 2*r*x );
    }

    return clamp(sum/(k-WarmUp), -0.99, 0);

  }//LyapunovExponent
  

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float l = lyapunovExponent(In.TexCd);
    float4 col = tex1D(Samp, l);
    return col;
}

float4 PS2(vs2ps In): COLOR
{
    float l = lyapunovExponentPS2(In.TexCd);
    float4 col = tex1D(Samp, l);
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TLyapunov
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}

technique TLyapunovPS2
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_b PS2();
    }
}
